/*
 * Copyright 2012, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.crypt;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import arlut.csd.crypto.MD5Crypt;
import arlut.csd.crypto.Sha256Crypt;
import arlut.csd.crypto.Sha512Crypt;

public enum Algorithm {

    CRYPT_MD5("^\\$1\\$.{9}.*") {
        @Override
        public String crypt(String toCrypt, String salt) {
            return MD5Crypt.crypt(toCrypt, salt);
        }
    },
    CRYPT_SHA256("^\\$5\\$.{16}.*") {
        @Override
        public String crypt(String toCrypt, String salt) throws CryptException {
            salt = salt.substring(3);
            String roundsString = getRoundsString(salt);
            int rounds = getRoundsCount(roundsString);
            return Sha256Crypt.Sha256_crypt(toCrypt, salt, rounds);
        }
    },
    CRYPT_SHA512("^\\$6\\$.{16}.*") {
        @Override
        public String crypt(String toCrypt, String salt) throws CryptException {
            salt = salt.substring(3);
            String roundsString = getRoundsString(salt);
            int rounds = getRoundsCount(roundsString);
            return Sha512Crypt.Sha512_crypt(toCrypt, salt, rounds);
        }
    },
    UNKNOWN("") {
        @Override
        public String crypt(String toCrypt, String salt) {
            return null;
        }
    };

    private static final int DEFAULT_ROUNDS = 5000;
    private static final int LOWEST_ROUNDS = 1000;
    private static final int HIGHEST_ROUNDS = 999999999;
    private static final String ROUNDS_START = "^rounds=";
    private static final String ROUNDS_END = "$";
    private static final Pattern roundsPattern = Pattern.compile(ROUNDS_START + "\\d+\\" + ROUNDS_END);

    private Pattern pattern;

    private Algorithm(String saltPattern) {
        pattern = Pattern.compile(saltPattern);
    }

    public static Algorithm getAlgorithm(String salt) {
        if (CRYPT_MD5.pattern.matcher(salt).matches()) {
            return CRYPT_MD5;
        }
        if (CRYPT_SHA256.pattern.matcher(salt).matches()) {
            return CRYPT_SHA256;
        }
        if (CRYPT_SHA512.pattern.matcher(salt).matches()) {
            return CRYPT_SHA512;
        }
        return UNKNOWN;
    }

    public abstract String crypt(String toCrypt, String salt) throws CryptException;

    private static int getRoundsCount(String roundsString) throws CryptException {
        roundsString = roundsString.replace(ROUNDS_START.substring(1), "");
        roundsString = roundsString.replace(ROUNDS_END, "");
        try {
            int i = Integer.parseInt(roundsString);
            i = Math.max(i, LOWEST_ROUNDS);
            i = Math.min(i, HIGHEST_ROUNDS);
            return i;
        } catch (NumberFormatException e) {
            return DEFAULT_ROUNDS;
        }
    }

    private static String getRoundsString(String salt) {
        Matcher matcher = roundsPattern.matcher(salt);
        if (matcher.find()) {
            return matcher.group();
        }
        return "";
    }

}
