package raging.goblin.infolog.android.sync;

import android.content.Context;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.Toast;

import org.apache.http.HttpVersion;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.HTTP;
import org.jasypt.exceptions.EncryptionOperationNotPossibleException;

import java.io.IOException;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.Date;

import raging.goblin.infolog.android.data.DbStorage;
import raging.goblin.infolog.android.gui.CurrentValuePreferencesActivity;
import raging.goblin.infolog.android.gui.R;
import raging.goblin.infolog.android.util.AccessDeniedException;
import raging.goblin.infolog.android.util.AndroidLogger;
import raging.goblin.infolog.android.util.CredentSSLSocketFactory;
import raging.goblin.infolog.android.util.JasyptCrypt;
import raging.goblin.infolog.sync.Synchronizer;
import raging.goblin.infolog.sync.log.Logger;
import raging.goblin.infolog.sync.webserviceclient.WebserviceClient;

public class SynchronizeActivity extends CurrentValuePreferencesActivity {

	private static final String LOG_TAG = "SynchronizeActivity";
	private static final Logger LOG = new AndroidLogger("Synchronizer");

	private Button syncButton;

	@Override
	protected void addPreferences() {
		addPreferencesFromResource(R.xml.syncpreferences);
		initSyncButton();
	}

	private void initSyncButton() {
		syncButton = new Button(this);
		syncButton.setText(getString(R.string.synchronize));
		syncButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				new Thread() {
					@Override
					public void run() {
						synchronize();
					}
				}.start();
			}
		});
		getListView().addFooterView(syncButton);
	}

	private void synchronize() {
		SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(this);

		String ip = preferences.getString(getString(R.string.key_sync_ip), "");
		int port = Integer.parseInt(preferences.getString(getString(R.string.key_sync_port), "0"));
		boolean https = preferences.getBoolean(getString(R.string.key_sync_https), false);
		WebserviceClient remoteStorage = new WebserviceClient(ip, port, https);

		DbStorage localStorage = new DbStorage(this);
		localStorage.start();

		String username = preferences.getString(getString(R.string.key_sync_username), "");
		String encryptionType = preferences.getString(getString(R.string.key_sync_encryptiontype), "");

		try {
			String password = JasyptCrypt.decryptPassword(preferences.getString(getString(R.string.key_sync_password), ""));
			Synchronizer synchronizer = new Synchronizer(LOG, localStorage, remoteStorage, username, password, encryptionType);

			remoteStorage.start(createNewHttpClient(preferences, port));

			if (!remoteStorage.authenticate(username, password, encryptionType)) {
				throw new AccessDeniedException();
			}

			showToast("Synchronization started, patience...");
			long lastSyncTime = Long.parseLong(preferences.getString(getString(R.string.key_last_sync_time), "0"));
			synchronizer.synchronize(lastSyncTime, true);

			SharedPreferences.Editor editor = preferences.edit();
			lastSyncTime = new Date().getTime();
			editor.putString(getString(R.string.key_last_sync_time), "" + lastSyncTime);
			editor.apply();

			showToast("Synchronization completed");
		} catch (EncryptionOperationNotPossibleException e) {
			Log.e(LOG_TAG, "Unable to decrypt password", e);
			showToast("Unable to retrieve password from device storage");
		} catch (AccessDeniedException e) {
			Log.e(LOG_TAG, "Unable to login on remote service", e);
			showToast("Unable to login on remote service with these settings");
		} catch (Exception e) {
			Log.e(LOG_TAG, "Syncing failed, aborting synchronization", e);
			showToast("Synchronization failed");
		}

		localStorage.stop();
		remoteStorage.stop();
	}

	private void showToast(final String message) {
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Context context = getApplicationContext();
				Toast toast = Toast.makeText(context, message, Toast.LENGTH_SHORT);
				toast.show();
			}
		});
	}

	private DefaultHttpClient createNewHttpClient(SharedPreferences preferences, int port) throws CertificateException, NoSuchAlgorithmException, IOException, UnrecoverableKeyException, KeyStoreException, KeyManagementException {
		boolean trustEveryBody = preferences.getBoolean(getString(R.string.key_sync_trust), false);
		if (trustEveryBody) {
			KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
			trustStore.load(null, null);

			SSLSocketFactory sf = new CredentSSLSocketFactory(trustStore);
			sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);

			HttpParams params = new BasicHttpParams();
			HttpProtocolParams.setVersion(params, HttpVersion.HTTP_1_1);
			HttpProtocolParams.setContentCharset(params, HTTP.UTF_8);

			SchemeRegistry registry = new SchemeRegistry();
			registry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), port));
			registry.register(new Scheme("https", sf, port));

			ClientConnectionManager ccm = new ThreadSafeClientConnManager(params, registry);

			return new DefaultHttpClient(ccm, params);
		}
		return new DefaultHttpClient();
	}

}
