/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.apache.http.impl.client.DefaultHttpClient;

import raging.goblin.crypt.EncryptionType;
import raging.goblin.crypt.JasyptCrypt;
import raging.goblin.infolog.client.gui.StickyNote;
import raging.goblin.infolog.client.properties.InfologClientProperties;
import raging.goblin.infolog.sync.data.Infolog;
import raging.goblin.infolog.sync.log.LogLevel;
import raging.goblin.infolog.sync.log.Logger;
import raging.goblin.infolog.sync.webserviceclient.WebserviceClient;

/**
 * The client is capable of getting in touch with the server and get the Infologs from the server. It also provides the
 * means to update the Infologs on the server.
 */
public class Client implements Observer {

	private static final Logger LOG = new Log4jLogger(Client.class);

	private JSONStorage jsonStorage;
	private List<StickyNote> stickyNotes;
	private int minInfologId = -1;
	private boolean notesVisible = false;
	private static final int HORIZONTAL_SPACING = 25;
	private RunnableSynchronizer synchronizer;

	public Client(JSONStorage jsonStorage) {
		this.jsonStorage = jsonStorage;
		stickyNotes = new ArrayList<StickyNote>();
		refreshStickyNotes();
	}

	public void update(final Infolog infolog) {
		new Thread() {
			@Override
			public void run() {
				List<Infolog> infologs = new ArrayList<>();
				infologs.add(infolog);
				jsonStorage.updateInfologs(infologs, null, null, null);
			}
		}.start();
	}

	public void add(final Infolog infolog) {
		new Thread() {
			@Override
			public void run() {
				List<Infolog> infologs = new ArrayList<>();
				infologs.add(infolog);
				jsonStorage.addInfologs(infologs, null, null, null);
			}
		}.start();
	}

	public void remove(final StickyNote stickyNote, final Infolog infolog) {
		stickyNotes.remove(stickyNote);
		new Thread() {
			@Override
			public void run() {
				List<Infolog> infologs = new ArrayList<>();
				infologs.add(infolog);
				jsonStorage.removeInfologs(infologs, null, null, null);
			}
		}.start();
	}

	public void layoutStickyNotes() {
		int x = InfologClientProperties.spacing;
		for (String type : new String[] { "note", "task", "phone" }) {
			x = layoutStickyNotesForType(type, x);
			x += InfologClientProperties.width + HORIZONTAL_SPACING;
		}
	}

	public void toggleVisibilityStickyNotes() {
		setNotesVisible(!notesVisible);
	}

	public void showNotes() {
		setNotesVisible(true);
	}

	public void newStickyNote(String type) {
		showNotes();
		Infolog newInfolog = createInfolog(type);
		add(newInfolog);
		StickyNote newNote = new StickyNote(newInfolog, notesVisible, this);
		stickyNotes.add(newNote);
	}

	@Override
	public void update(Observable o, Object arg) {
		refreshStickyNotes();
	}

	public boolean tryRemoteLogin(String hostname, int port, String username, String password, boolean https,
	   EncryptionType encryptionType) {
		if (!username.equals("") && !password.equals("")) {
			WebserviceClient webserviceClient = createRemoteStorage(hostname, port, https);
			if (webserviceClient != null) {
				boolean success = false;
				webserviceClient.start(new DefaultHttpClient());
				try {
					success = webserviceClient.authenticate(username, password, encryptionType.toString());
					webserviceClient.stop();
					if (success) {
						LOG.log(LogLevel.DEBUG, "Successfuly logging in on remote server");
						return true;
					} else {
						LOG.log(LogLevel.WARN, "Failed to log in on remote server with these credentials");
					}
				} catch (Exception e) {
					LOG.log(LogLevel.ERROR, "Unable to authenticate", e);
				}
			}
		}
		return false;
	}

	public void refreshStickyNotes() {
		disposeStickyNotes();
		stickyNotes = new ArrayList<StickyNote>();
		for (Infolog infolog : jsonStorage.getInfologs(null, null, null)) {
			stickyNotes.add(new StickyNote(infolog, notesVisible, this));
			if (infolog.getId() < minInfologId) {
				minInfologId = infolog.getId();
			}
		}
		layoutStickyNotes();
	}

	public void startSynchronizing() {
		stopSynchronizing();

		if (isSynchronizationEnabled()) {
			new Thread() {
				@Override
				public void run() {
					if (tryRemoteLogin(InfologClientProperties.serverHostName, InfologClientProperties.serverPort,
					   InfologClientProperties.username, JasyptCrypt.decryptPassword(InfologClientProperties.password),
					   InfologClientProperties.https, EncryptionType
					      .getEncryptionType(InfologClientProperties.encryptionType))) {
						synchronizer = new RunnableSynchronizer(jsonStorage, createRemoteStorage(
						   InfologClientProperties.serverHostName, InfologClientProperties.serverPort,
						   InfologClientProperties.https));
						synchronizer.addObserver(Client.this);
						synchronizer.start();
					} else {
						LOG.log(LogLevel.WARN, "Not able to login on the remote server, synchonization not started");
					}
				}
			}.start();
		}
	}

	public void stopSynchronizing() {
		if (synchronizer != null) {
			synchronizer.stop();
		}
	}

	public void synchronize() {
		new Thread() {
			@Override
			public void run() {
				if (tryRemoteLogin(InfologClientProperties.serverHostName, InfologClientProperties.serverPort,
				   InfologClientProperties.username, JasyptCrypt.decryptPassword(InfologClientProperties.password),
				   InfologClientProperties.https, EncryptionType.getEncryptionType(InfologClientProperties.encryptionType))) {
					WebserviceClient webserviceClient = createRemoteStorage(InfologClientProperties.serverHostName,
					   InfologClientProperties.serverPort, InfologClientProperties.https);
					synchronizer = new RunnableSynchronizer(jsonStorage, webserviceClient);
					synchronizer.addObserver(Client.this);
					try {
						webserviceClient.start(new DefaultHttpClient());
						synchronizer.synchronize();
						webserviceClient.stop();
					} catch (Exception e) {
						LOG.log(LogLevel.ERROR, "Unable to synchronize, aborting synhronization...", e);
					}
				}
			}
		}.start();
	}

	private int layoutStickyNotesForType(String type, int x) {
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension resolution = toolkit.getScreenSize();
		int y = InfologClientProperties.spacing;
		for (StickyNote stickyNote : stickyNotes) {
			if (stickyNote.getInfologType().equals(type)) {
				stickyNote.setLocation(x, y);
				y += InfologClientProperties.spacing;
				if (y + InfologClientProperties.height >= resolution.getHeight() - InfologClientProperties.spacing) {
					y = InfologClientProperties.spacing;
					x += InfologClientProperties.width + HORIZONTAL_SPACING;
					if (x + InfologClientProperties.width >= resolution.getWidth() - InfologClientProperties.spacing) {
						x = InfologClientProperties.spacing;
						y = InfologClientProperties.spacing;
					}
				}
			}
		}
		return x;
	}

	private Infolog createInfolog(String type) {
		return new Infolog(--minInfologId, type);
	}

	private void setNotesVisible(boolean visible) {
		notesVisible = visible;
		for (StickyNote stickyNote : stickyNotes) {
			stickyNote.setVisible(notesVisible);
		}
	}

	private void disposeStickyNotes() {
		for (StickyNote stickyNote : stickyNotes) {
			stickyNote.dispose();
		}
	}

	private boolean isSynchronizationEnabled() {
		return InfologClientProperties.pollInterval > 0;
	}

	private WebserviceClient createRemoteStorage(String hostname, int port, boolean useHttps) {
		if (useHttps) {
			if (!loadKeyStore(hostname, port)) {
				return null;
			}
		}

		WebserviceClient remoteStorage = new WebserviceClient(hostname, port, useHttps);
		return remoteStorage;
	}

	private boolean loadKeyStore(String hostName, int port) {
		File keyStore = new File(InfologClientProperties.keystore);

		if (createKeyStore(hostName, port, keyStore, InfologClientProperties.passphrase.toCharArray())) {
			System.setProperty("javax.net.ssl.trustStore", keyStore.getAbsolutePath());
			System.setProperty("javax.net.ssl.trustStorePassword", InfologClientProperties.passphrase);
			return true;
		}

		return false;
	}

	private boolean createKeyStore(String hostName, int port, File keyStore, char[] passphrase) {
		if (keyStore.isFile()) {
			return true;
		}

		try {
			return InfologKeyStoreCreator.createKeyStore(hostName, port, keyStore, passphrase);
		} catch (Exception e) {
			LOG.log(LogLevel.ERROR, "Unable to import certificate", e);
		}

		return false;
	}
}
