/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;

import raging.goblin.infolog.sync.InfologStorage;
import raging.goblin.infolog.sync.data.Infolog;

/**
 * Specific implementation of the InfologStorage interface to be used locally. It stores the {@link Infolog} objects in
 * an JSON file.
 */
public class JSONStorage implements InfologStorage {

	private static Logger LOG = Logger.getLogger(JSONStorage.class);

	private static final String SYNC_METADATA_FILE = "data/sync.json";
	private static final String INFOLOG_DATA_FILE = "data/infologs.json";

	private List<Infolog> infologs;
	private ObjectMapper mapper;

	public JSONStorage() {
		mapper = new ObjectMapper();
		infologs = unmarshallInfologs();
		if (infologs == null) {
			infologs = new ArrayList<Infolog>();
		}
	}

	@Override
	public List<Infolog> addInfologs(List<Infolog> infologs, String username, String password, String encryptionType) {
		this.infologs.addAll(infologs);
		marshallInfologs();
		return this.infologs;
	}

	@Override
	public boolean authenticate(String username, String password, String encryptionType) {
		return true;
	}

	@Override
	public List<Infolog> getInfologs(String username, String password, String encryptionType) {
		return new ArrayList<Infolog>(infologs);
	}

	@Override
	public void removeInfologs(List<Infolog> infologs, String username, String password, String encryptionType) {
		this.infologs.removeAll(infologs);
		marshallInfologs();
	}

	@Override
	public void updateInfologs(List<Infolog> infologs, String username, String password, String encryptionType) {
		this.infologs.removeAll(infologs);
		this.infologs.addAll(infologs);
		marshallInfologs();
	}

	public void setLastSyncTime(long lastSyncTime) {
		try {
			mapper.writeValue(new File(SYNC_METADATA_FILE), lastSyncTime);
		} catch (IOException e) {
			LOG.error("Unable to write sync info", e);
		}
	}

	public long getLastSyncTime() {
		try {
			return mapper.readValue(new File(SYNC_METADATA_FILE), new TypeReference<Long>() {
			});
		} catch (IOException e) {
			LOG.error("Unable to read sync info", e);
		}
		return 0;
	}

	private List<Infolog> unmarshallInfologs() {
		try {
			return mapper.readValue(new File(INFOLOG_DATA_FILE), new TypeReference<List<Infolog>>() {
			});
		} catch (IOException e) {
			LOG.error("Unable to Unmarshall infologs", e);
		}
		return null;
	}

	private void marshallInfologs() {
		try {
			mapper.writeValue(new File(INFOLOG_DATA_FILE), infologs);
		} catch (IOException e) {
			LOG.error("Unable to marshall infologs", e);
		}
	}

}
