/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Date;
import java.util.Observable;

import javax.swing.SwingUtilities;

import org.apache.http.impl.client.DefaultHttpClient;
import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;

import raging.goblin.crypt.JasyptCrypt;
import raging.goblin.infolog.client.properties.InfologClientProperties;
import raging.goblin.infolog.sync.Synchronizer;
import raging.goblin.infolog.sync.data.Infolog;
import raging.goblin.infolog.sync.log.LogLevel;
import raging.goblin.infolog.sync.log.Logger;
import raging.goblin.infolog.sync.webserviceclient.HttpErrorCodeException;
import raging.goblin.infolog.sync.webserviceclient.WebserviceClient;

/**
 * The RunnableSynchronizer synchronizes the local {@link Infolog} objects with the ones on the server on a regular
 * interval.
 */
public class RunnableSynchronizer extends Observable implements Runnable {

    private static final Logger logger = new Log4jLogger(RunnableSynchronizer.class);

    private JSONStorage localStorage;
    private WebserviceClient remoteStorage;
    private volatile boolean running = false;

    public RunnableSynchronizer(JSONStorage localStorage, WebserviceClient remoteStorage) {
        this.localStorage = localStorage;
        this.remoteStorage = remoteStorage;
    }

    public void start() {
        running = true;
        remoteStorage.start(new DefaultHttpClient());
        new Thread(this).start();
        logger.log(LogLevel.INFO, "Synchronization started");
    }

    public void stop() {
        running = false;
        remoteStorage.stop();
        logger.log(LogLevel.INFO, "Synchronization stopped");
    }

    public void synchronize() throws JsonParseException, JsonMappingException, IOException, HttpErrorCodeException,
        URISyntaxException {
        Synchronizer synchronizer = new Synchronizer(new Log4jLogger(Synchronizer.class), localStorage, remoteStorage,
            InfologClientProperties.username, JasyptCrypt.decryptPassword(InfologClientProperties.password),
            InfologClientProperties.encryptionType);
        synchronizer.synchronize(localStorage.getLastSyncTime(), InfologClientProperties.serverWins);
        updateSyncInfo();
        updateClient();
        logger.log(LogLevel.INFO, "Synchronization complete, client updated.");
    }

    private void updateSyncInfo() {
        long lastSync = new Date().getTime();
        localStorage.setLastSyncTime(lastSync);
    }

    private void updateClient() {
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                setChanged();
                notifyObservers();
            }
        });
    }

    @Override
    public void run() {
        while (running) {
            try {
                synchronize();
                try {
                    Thread.sleep(InfologClientProperties.pollInterval * 60 * 1000);
                } catch (InterruptedException e) {
                    logger.log(LogLevel.WARN, "Synchronizing interupted");
                    logger.log(LogLevel.DEBUG, "RunnableSynchronizer interupted", e);
                }
            } catch (Exception e) {
                logger.log(LogLevel.ERROR, "Unable to synchronize, aborting synhronization...", e);
                running = false;
            }
        }
    }
}
