/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client.gui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.SystemTray;
import java.awt.Toolkit;
import java.awt.TrayIcon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.Timer;

import org.apache.log4j.Logger;

import raging.goblin.infolog.client.Client;
import raging.goblin.infolog.client.properties.InfologClientProperties;
import raging.goblin.infolog.sync.data.Infolog;

/**
 * The ClientGui creates a new window to login and to show, hide, delete an create new StickyNotes that represent the
 * Infolog objects on the server.
 */
public class ClientGui extends JFrame {

	private static final String CLIENT_TITLE = "Infolog client";
	private static final String ABOUT_TITLE = "About Infolog";
	private static final String ABOUT_LOGO = "icons/infolog.png";
	private static final String WINDOW_ICON = "icons/infolog.png";
	private static final int ICON_GAP = 10;
	private static final int CENTER_GAP = 40;

	private static final ImageIcon EXIT_ICON = new ImageIcon("icons/cross.png");
	private static final ImageIcon SYNC_CONFIG_ICON = new ImageIcon("icons/database_edit.png");
	private static final ImageIcon GUI_CONFIG_ICON = new ImageIcon("icons/wrench.png");
	private static final ImageIcon LAYOUT_ICON = new ImageIcon("icons/application_cascade.png");
	private static final ImageIcon NOTE_ICON = new ImageIcon("icons/note_add.png");
	private static final ImageIcon TASK_ICON = new ImageIcon("icons/time_add.png");
	private static final ImageIcon PHONE_ICON = new ImageIcon("icons/telephone_add.png");
	private static final ImageIcon REFRESH_ICON = new ImageIcon("icons/arrow_rotate_clockwise.png");
	private static final ImageIcon SYNC_ICON = new ImageIcon("icons/database_refresh.png");
	private static final ImageIcon VISIBILITY_ICON = new ImageIcon("icons/Application_view_tile.png");
	private static final ImageIcon HELP_ICON = new ImageIcon("icons/help.png");
	private static final ImageIcon ABOUT_ICON = new ImageIcon("icons/star.png");
	private static final ImageIcon SHOW_GUI_ICON = new ImageIcon("icons/application_form_magnify.png");
	private static final String ABOUT_TEXT = "<html><h1>Infolog desktop client</h1>"
	   + "<i>Version 4.0, December 2013<br />http://raginggoblin.wordpress.com/infolog</i><br /><br />"
	   + "Infolog desktop client is the desktop companion of Egroupware Infolog.<br />"
	   + "See the Egroupware website (<i>http://www.egroupware.org</i>) for more <br />"
	   + "information about Egroupware or Infolog.<br /><br />"
	   + "Infolog desktop client is released under the <i>Gnu Public License</i>.</html>";

	private static Logger logger = Logger.getLogger("raging.goblin.infolog.client.ClientGui");

	private JPopupMenu systrayMenu;
	private Timer clickTimer;
	private TrayIcon trayIcon;
	private double maxPreferredWidth;
	private JButton newNoteButton;
	private JButton newTaskButton;
	private JButton newPhoneButton;
	private JMenuItem newNoteMenuItem;
	private JMenuItem newTaskMenuItem;
	private JMenuItem newPhoneMenuItem;
	private JMenuItem newNoteSystrayItem;
	private JMenuItem newTaskSystrayItem;
	private JMenuItem newPhoneSystrayItem;
	private Client client;
	private boolean trayMenuVisible = false;

	private ActionListener refreshListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			client.showNotes();
			client.refreshStickyNotes();
		}
	};

	private ActionListener syncListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			client.showNotes();
			client.synchronize();
		}
	};

	private ActionListener toggleListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			client.toggleVisibilityStickyNotes();
		}
	};

	private ActionListener newNoteListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			client.newStickyNote(Infolog.NOTE);
		}
	};

	private ActionListener newPhoneListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			client.newStickyNote(Infolog.PHONE);
		}
	};

	private ActionListener newTaskListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			client.newStickyNote(Infolog.TASK);
		}
	};

	private ActionListener quitListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			System.exit(NORMAL);
		}
	};

	private ActionListener layoutListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			client.layoutStickyNotes();
		}
	};

	public ClientGui(Client client) {
		super(CLIENT_TITLE);
		this.client = client;
		setDefaultCloseOperation(SystemTray.isSupported() ? HIDE_ON_CLOSE : EXIT_ON_CLOSE);
		setIconImage(Toolkit.getDefaultToolkit().getImage(WINDOW_ICON));
		initGui();
		loadSystray();
	}

	private void initGui() {
		add(createClientActionsPanel(), BorderLayout.WEST);
		add(createInfologActionsPanel(), BorderLayout.EAST);
		add(new JPanel(), BorderLayout.CENTER);

		JButton toggleButton = new JButton("Toggle visibility", VISIBILITY_ICON);
		toggleButton.addActionListener(toggleListener);
		add(toggleButton, BorderLayout.SOUTH);

		setJMenuBar(createMenu());

		setSize((int) (2 * maxPreferredWidth) + CENTER_GAP, 190);
	}

	private void loadSystray() {
		if (SystemTray.isSupported()) {
			try {
				SystemTray tray = SystemTray.getSystemTray();
				Image icon = Toolkit.getDefaultToolkit().getImage(InfologClientProperties.trayIcon);
				trayIcon = new TrayIcon(icon);
				trayIcon.addMouseListener(new TrayIconMouseListener());
				tray.add(trayIcon);
				systrayMenu = createSystrayMenu();
			} catch (Exception e) {
				logger.warn("TrayIcon could not be added.");
				logger.debug(e.getMessage(), e);
				setVisible(true);
			}
		} else {
			setVisible(true);
		}
	}

	private JPopupMenu createSystrayMenu() {
		JPopupMenu menu = new JPopupMenu();

		JMenuItem showGuiItem = new JMenuItem("Show gui", SHOW_GUI_ICON);
		showGuiItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				trayMenuVisible = false;
				setVisible(true);
			}
		});
		menu.add(showGuiItem);

		menu.addSeparator();

		newNoteSystrayItem = new JMenuItem("New Note", NOTE_ICON);
		newNoteSystrayItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				trayMenuVisible = false;
				client.newStickyNote(Infolog.NOTE);
			}
		});
		menu.add(newNoteSystrayItem);

		newTaskSystrayItem = new JMenuItem("New Task", TASK_ICON);
		newTaskSystrayItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				trayMenuVisible = false;
				client.newStickyNote(Infolog.TASK);
			}
		});
		menu.add(newTaskSystrayItem);

		newPhoneSystrayItem = new JMenuItem("New Phone Call", PHONE_ICON);
		newPhoneSystrayItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				trayMenuVisible = false;
				client.newStickyNote(Infolog.PHONE);
			}
		});
		menu.add(newPhoneSystrayItem);

		menu.addSeparator();

		JMenuItem quitItem = new JMenuItem("Quit", EXIT_ICON);
		quitItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				System.exit(NORMAL);
			}
		});
		menu.add(quitItem);

		return menu;
	}

	private JMenuBar createMenu() {
		JMenuBar menuBar = new JMenuBar();

		JMenu fileMenu = new JMenu("File");

		newNoteMenuItem = new JMenuItem("New Note", NOTE_ICON);
		newNoteMenuItem.addActionListener(newNoteListener);
		fileMenu.add(newNoteMenuItem);

		newTaskMenuItem = new JMenuItem("New Task", TASK_ICON);
		newTaskMenuItem.addActionListener(newTaskListener);
		fileMenu.add(newTaskMenuItem);

		newPhoneMenuItem = new JMenuItem("New Phone Call", PHONE_ICON);
		newPhoneMenuItem.addActionListener(newPhoneListener);
		fileMenu.add(newPhoneMenuItem);

		fileMenu.addSeparator();

		JMenuItem quitItem = new JMenuItem("Quit", EXIT_ICON);
		quitItem.addActionListener(quitListener);
		fileMenu.add(quitItem);

		menuBar.add(fileMenu);

		JMenu actionsMenu = new JMenu("Actions");

		JMenuItem configureSyncItem = new JMenuItem("Configure synchronization", SYNC_CONFIG_ICON);
		configureSyncItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				showSyncConfigDialog();
			}
		});
		actionsMenu.add(configureSyncItem);

		JMenuItem configureGuiItem = new JMenuItem("Configure look and feel", GUI_CONFIG_ICON);
		configureGuiItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				showGuiConfigDialog();
			}
		});
		actionsMenu.add(configureGuiItem);

		actionsMenu.addSeparator();

		JMenuItem syncItem = new JMenuItem("Synchronize", SYNC_ICON);
		syncItem.addActionListener(syncListener);
		actionsMenu.add(syncItem);

		actionsMenu.addSeparator();

		JMenuItem refreshItem = new JMenuItem("Refresh", REFRESH_ICON);
		refreshItem.addActionListener(refreshListener);
		actionsMenu.add(refreshItem);

		JMenuItem layoutItem = new JMenuItem("Layout", LAYOUT_ICON);
		layoutItem.addActionListener(layoutListener);
		actionsMenu.add(layoutItem);

		JMenuItem toggleItem = new JMenuItem("Toggle visibility", VISIBILITY_ICON);
		toggleItem.addActionListener(toggleListener);
		actionsMenu.add(toggleItem);

		menuBar.add(actionsMenu);

		JMenu helpMenu = new JMenu("Help");

		JMenuItem helpItem = new JMenuItem("Help", HELP_ICON);
		helpItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				showHelpDialog();
			}
		});
		helpMenu.add(helpItem);

		JMenuItem aboutItem = new JMenuItem("About", ABOUT_ICON);
		aboutItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				showAboutInfo();
			}
		});
		helpMenu.add(aboutItem);

		menuBar.add(helpMenu);

		return menuBar;
	}

	private JPanel createClientActionsPanel() {
		JPanel actionsPanel = new JPanel(new GridLayout(0, 1, 10, 5));

		JButton syncButton = new JButton("Synchronize", SYNC_ICON);
		maxPreferredWidth = syncButton.getPreferredSize().getWidth();
		syncButton.setIconTextGap(ICON_GAP);
		syncButton.addActionListener(syncListener);
		actionsPanel.add(syncButton);

		JButton refreshButton = new JButton("Refresh", REFRESH_ICON);
		refreshButton.setIconTextGap(ICON_GAP);
		refreshButton.addActionListener(refreshListener);
		actionsPanel.add(refreshButton);

		JButton layoutButton = new JButton("Layout", LAYOUT_ICON);
		layoutButton.setIconTextGap(ICON_GAP);
		layoutButton.addActionListener(layoutListener);
		actionsPanel.add(layoutButton);

		return actionsPanel;
	}

	private JPanel createInfologActionsPanel() {
		JPanel actionsPanel = new JPanel(new GridLayout(0, 1, 10, 5));

		newNoteButton = new JButton("Note", NOTE_ICON);
		newNoteButton.addActionListener(newNoteListener);
		actionsPanel.add(newNoteButton);

		newTaskButton = new JButton("Task", TASK_ICON);
		newTaskButton.addActionListener(newTaskListener);
		actionsPanel.add(newTaskButton);

		newPhoneButton = new JButton("Phone Call", PHONE_ICON);
		maxPreferredWidth = Math.max(maxPreferredWidth, newPhoneButton.getPreferredSize().getWidth());
		newPhoneButton.setMargin(new Insets(10, 10, 10, 10));
		newPhoneButton.addActionListener(newPhoneListener);
		actionsPanel.add(newPhoneButton);

		return actionsPanel;
	}

	private void showHelpDialog() {
		HelpDialog helpFrame = new HelpDialog(this);
		helpFrame.setVisible(true);
	}

	private void showSyncConfigDialog() {
		SyncConfigDialog dialog = new SyncConfigDialog(this, client);
		dialog.dispose();
	}

	private void showGuiConfigDialog() {
		GuiConfigDialog dialog = new GuiConfigDialog(this);
		dialog.dispose();
		client.refreshStickyNotes();
	}

	private void showAboutInfo() {
		JOptionPane.showMessageDialog(this, ABOUT_TEXT, ABOUT_TITLE, JOptionPane.YES_OPTION, new ImageIcon(ABOUT_LOGO));
	}

	private class TrayIconMouseListener implements MouseListener {

		@Override
		public void mouseClicked(MouseEvent e) {
			// Left button
			if (e.getButton() != MouseEvent.BUTTON3) {
				trayMenuVisible = false;
				if (e.getClickCount() == 1) {
					// Single click - toggle visibility of sticky notes
					clickTimer = new Timer(InfologClientProperties.doubleClickDelay, toggleListener);
					clickTimer.setRepeats(false);
					clickTimer.start();
				} else if (e.getClickCount() >= 2) {
					// Double click - show/hide gui
					clickTimer.stop();
					setVisible(!isVisible());
				}
			}
		}

		@Override
		public void mouseEntered(MouseEvent e) {
			// Nothing to do
		}

		@Override
		public void mouseExited(MouseEvent e) {
			// Nothing to do
		}

		@Override
		public void mousePressed(MouseEvent e) {
			if (e.getButton() == MouseEvent.BUTTON3) {
				// Right button - show popup menu
				trayMenuVisible = !trayMenuVisible;
				systrayMenu.setLocation(e.getX() + 1, e.getY() + 1);
				systrayMenu.setInvoker(systrayMenu);
				systrayMenu.setVisible(trayMenuVisible);
			}
		}

		@Override
		public void mouseReleased(MouseEvent e) {
			// Nothing to do
		}
	}

}
