/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.MouseInputAdapter;

import raging.goblin.infolog.client.Client;
import raging.goblin.infolog.client.properties.InfologClientProperties;
import raging.goblin.infolog.sync.data.Infolog;

/**
 * This Class represents a sticky note, these yellow notes on which you can scribble something you want to remember. It
 * appears as a coloured note on your screen containing a title bar with the title you define, a date displayed at the
 * bottom and a text box which you can fill with text.
 */
public class StickyNote extends JFrame implements DocumentListener {

	private static final int BORDERWIDTH = 4;
	private static final int GLUE_DISCOUNT = 20;

	private int dragCounter = 0;
	private int dragInitialX;
	private int dragInitialY;
	private JTextArea subjectField = new JTextArea();
	private JTextArea descriptionArea = new JTextArea();
	private JLabel timestampLabel = new JLabel();
	private DateFormat format = new SimpleDateFormat("dd-MM-yyyy HH:mm");
	private Infolog infolog;
	private Client client;

	public StickyNote(String subject, String description, long timestamp, boolean visible, String type) {
		super(subject);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setUndecorated(true);
		setSize(InfologClientProperties.width, InfologClientProperties.height);
		initIcon(type);
		initGui(type);
		setSubject(subject);
		setDescription(description);
		setTimestamp(timestamp);
		setVisible(visible);
		initDocumentListeners();
	}

	public StickyNote(Infolog infolog, boolean visible, Client client) {
		this(infolog.getSubject(), infolog.getDescription(), infolog.getDatemodified(), visible, infolog.getType());
		this.infolog = infolog;
		this.client = client;
	}

	public static int getMinimumColor(int colorValue) {
		int glueColor = colorValue - GLUE_DISCOUNT;
		return Math.max(glueColor, 0);
	}

	private void initGui(String type) {
		int[] topColor;
		int[] bottomColor;
		if (type.equals(Infolog.PHONE)) {
			topColor = InfologClientProperties.phoneTopColor;
			bottomColor = InfologClientProperties.phoneBottomColor;
		} else if (type.equals(Infolog.TASK)) {
			topColor = InfologClientProperties.taskTopColor;
			bottomColor = InfologClientProperties.taskBottomColor;
		} else {
			topColor = InfologClientProperties.noteTopColor;
			bottomColor = InfologClientProperties.noteBottomColor;
		}
		setContentPane(new Background(topColor[0], topColor[1], topColor[2], bottomColor[0], bottomColor[1],
		   bottomColor[2]));
		getContentPane().setLayout(new BorderLayout());

		JPanel topPanel = new JPanel(new BorderLayout());
		topPanel.setOpaque(true);
		Color glueColor = new Color(getMinimumColor(topColor[0]), getMinimumColor(topColor[1]),
		   getMinimumColor(topColor[2]));
		topPanel.setBackground(glueColor);

		DragAdapter dragger = new DragAdapter();
		JLabel dragLabel = new JLabel(" ");
		dragLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 5));
		dragLabel.setOpaque(false);
		dragLabel.addMouseListener(dragger);
		dragLabel.addMouseMotionListener(dragger);
		topPanel.add(dragLabel, BorderLayout.NORTH);

		topPanel.add(subjectField, BorderLayout.CENTER);
		subjectField.setRows(1);
		subjectField.setLineWrap(true);
		subjectField.setFont(subjectField.getFont().deriveFont(Font.BOLD));
		subjectField.setOpaque(false);
		subjectField.setBorder(BorderFactory.createEmptyBorder(BORDERWIDTH, BORDERWIDTH, BORDERWIDTH, BORDERWIDTH));

		add(topPanel, BorderLayout.NORTH);

		add(descriptionArea, BorderLayout.CENTER);
		descriptionArea.setLineWrap(true);
		descriptionArea.setOpaque(false);
		descriptionArea.setBorder(BorderFactory.createEmptyBorder(BORDERWIDTH, BORDERWIDTH, BORDERWIDTH, BORDERWIDTH));

		JLabel deleteLabel = new JLabel("X");
		deleteLabel.setToolTipText("Delete");
		deleteLabel.setFont(deleteLabel.getFont().deriveFont(Font.BOLD));
		deleteLabel.setBorder(BorderFactory.createEmptyBorder(BORDERWIDTH, BORDERWIDTH, BORDERWIDTH, BORDERWIDTH));
		deleteLabel.setOpaque(false);
		deleteLabel.addMouseListener(new DeleteAdapter());
		topPanel.add(deleteLabel, BorderLayout.EAST);

		add(timestampLabel, BorderLayout.SOUTH);
		timestampLabel.setFont(timestampLabel.getFont().deriveFont(Font.BOLD));
		timestampLabel.setOpaque(false);
		timestampLabel.setHorizontalAlignment(SwingConstants.RIGHT);
		timestampLabel.setBorder(BorderFactory.createEmptyBorder(BORDERWIDTH, BORDERWIDTH, BORDERWIDTH, BORDERWIDTH));
		ResizeAdapter resizer = new ResizeAdapter();
		timestampLabel.addMouseListener(resizer);
		timestampLabel.addMouseMotionListener(resizer);
	}

	private void initIcon(String type) {
		if (type.equals(Infolog.PHONE)) {
			setIconImage(StickyNoteIcon.getPhoneIcon());
		} else if (type.equals(Infolog.TASK)) {
			setIconImage(StickyNoteIcon.getTaskIcon());
		} else {
			setIconImage(StickyNoteIcon.getNoteIcon());
		}
	}

	private void initDocumentListeners() {
		descriptionArea.getDocument().addDocumentListener(this);
		subjectField.getDocument().addDocumentListener(this);
	}

	public String getInfologType() {
		return infolog.getType();
	}

	public void setTimestamp(long timestamp) {
		timestampLabel.setText(format.format(new Date(timestamp)));
	}

	public void setDescription(String description) {
		descriptionArea.setText(description);
	}

	public void setSubject(String subject) {
		subjectField.setText(subject);
		setTitle(subject);
	}

	@Override
	public void changedUpdate(DocumentEvent arg0) {
		updateInfolog();
	}

	@Override
	public void insertUpdate(DocumentEvent arg0) {
		updateInfolog();
	}

	@Override
	public void removeUpdate(DocumentEvent arg0) {
		updateInfolog();
	}

	private void setPositionX(int x) {
		setBounds(x, getY(), getWidth(), getHeight());
	}

	private void setPositionY(int y) {
		setBounds(getX(), y, getWidth(), getHeight());
	}

	private void updateInfolog() {
		if (!subjectField.getText().equals(infolog.getSubject())
		   || !descriptionArea.getText().equals(infolog.getDescription())) {
			infolog.setSubject(subjectField.getText());
			setTitle(subjectField.getText());
			infolog.setDescription(descriptionArea.getText());
			long changed = new Date().getTime();
			infolog.setDatemodified(changed);
			setTimestamp(changed);
			client.update(infolog);
		}
	}

	/**
	 * The DeleteAdapter provides the mouse functionality for the deleteLabel.
	 */
	private class DeleteAdapter extends MouseAdapter {

		@Override
		public void mousePressed(MouseEvent e) {
			client.remove(StickyNote.this, infolog);
			setVisible(false);
			dispose();
		}

		@Override
		public void mouseExited(MouseEvent e) {
			Cursor cursor = new Cursor(Cursor.DEFAULT_CURSOR);
			setCursor(cursor);
		}

		@Override
		public void mouseEntered(MouseEvent e) {
			Cursor cursor = new Cursor(Cursor.HAND_CURSOR);
			setCursor(cursor);
		}
	}

	/**
	 * The ResizeAdapter provides the mouse functionality to resize the sticky note. It is supposed to be in the right
	 * lower corner.
	 */
	private class ResizeAdapter extends MouseInputAdapter {

		@Override
		public void mouseEntered(MouseEvent e) {
			Cursor cursor = new Cursor(Cursor.SE_RESIZE_CURSOR);
			setCursor(cursor);
		}

		@Override
		public void mouseDragged(MouseEvent e) {
			StickyNote.this.setBounds(getX(), getY(), e.getXOnScreen() - getX(), e.getYOnScreen() - getY());
		}

		@Override
		public void mouseExited(MouseEvent e) {
			Cursor cursor = new Cursor(Cursor.DEFAULT_CURSOR);
			setCursor(cursor);
		}
	}

	/**
	 * The DragAdapter provides the mouse functionality for dragging the StickyNote around.
	 */
	private class DragAdapter extends MouseInputAdapter {

		@Override
		public void mousePressed(MouseEvent e) {
			dragInitialX = e.getXOnScreen() - getX();
			dragInitialY = e.getYOnScreen() - getY();
		}

		@Override
		public void mouseDragged(MouseEvent e) {
			dragCounter++;
			if (dragCounter > InfologClientProperties.dragDelay) {
				setPositionX(e.getXOnScreen() - dragInitialX);
				setPositionY(e.getYOnScreen() - dragInitialY);
				dragCounter = 0;
			}
		}

		@Override
		public void mouseEntered(MouseEvent e) {
			Cursor cursor = new Cursor(Cursor.MOVE_CURSOR);
			setCursor(cursor);
		}

		@Override
		public void mouseExited(MouseEvent e) {
			Cursor cursor = new Cursor(Cursor.DEFAULT_CURSOR);
			setCursor(cursor);
		}
	}

	/**
	 * The Background class is a specialised JPanel that provides a background with a gradient colour.
	 */
	private class Background extends JPanel {

		private Color topColor;
		private Color bottomColor;

		public Background(int rTop, int gTop, int bTop, int rBottom, int gBottom, int bBottom) {
			topColor = new Color(rTop, gTop, bTop);
			bottomColor = new Color(rBottom, gBottom, bBottom);
		}

		@Override
		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			Graphics2D g2 = (Graphics2D) g.create();
			GradientPaint gradient = new GradientPaint(0, 0, topColor, getWidth(), this.getHeight(), bottomColor);
			g2.setPaint(gradient);
			g2.fillRect(0, 0, getWidth(), getHeight());
		}
	}
}
