/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.client.properties;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

/**
 * The Properties class provides an easy way to load and save the Infolog configuration file. This class has its own way
 * of handling properties because the standard java Properties class cannot handle complex comment structures. It
 * implements the Singleton design pattern.
 */
public class Properties {

	private static final String FILE = "infolog.properties";
	private static final String COMMENT = "#";
	private static final String ASSIGNMENT = "=";

	private static Logger LOG = Logger.getLogger("egroupware.infolog.InfologProperties");

	private static Properties instance = null;

	private Map<String, String> properties = new HashMap<String, String>();
	private Map<String, Integer> propertyLines = new HashMap<String, Integer>();
	private List<String> lines = new ArrayList<String>();

	private Properties() {
		loadProperties();
	}

	public static Properties getInstance() {
		if (instance == null) {
			instance = new Properties();
		}
		return instance;
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		throw new CloneNotSupportedException("No cloning of Singleton allowed");
	}

	public String getProperty(String key) {
		return properties.get(key);
	}

	public String getProperty(String key, String defaultValue) {
		String value = properties.get(key);
		if (value == null) {
			return defaultValue;
		}
		return value;
	}

	public void setProperty(String key, String value) {
		key = key.trim();
		value = value.trim();
		if (properties.keySet().contains(key) || propertyLines.keySet().contains(key)) {
			int propertyLine = propertyLines.get(key);
			properties.remove(key);
			propertyLines.remove(key);
			lines.remove(propertyLine);
			properties.put(key, value);
			propertyLines.put(key, propertyLine);
			lines.add(propertyLine, key + " = " + value);
		} else {
			properties.put(key, value);
			propertyLines.put(key, lines.size());
			lines.add(key + " = " + value);
		}
	}

	public void save() {
		try (PrintWriter writer = new PrintWriter(new FileWriter(FILE))) {
			for (String line : lines) {
				writer.println(line);
			}
		} catch (IOException e) {
			LOG.error("Unable to write to properties file: " + FILE);
			LOG.debug(e.getMessage(), e);
		}
	}

	private void loadProperties() {
		try (BufferedReader reader = new BufferedReader(new FileReader(FILE))) {
			String line = null;
			int lineNumber = 0;
			while ((line = reader.readLine()) != null) {
				lines.add(line);
				if (!line.startsWith(COMMENT) && line.contains(ASSIGNMENT)) {
					List<String> keyValuePair = getKeyValuePair(line);
					propertyLines.put(keyValuePair.get(0), lineNumber);
					properties.put(keyValuePair.get(0), keyValuePair.get(1));
				}
				lineNumber++;
			}
		} catch (IOException e) {
			LOG.error("Error reading from properties file: " + FILE);
			LOG.debug(e.getMessage(), e);
		}
	}

	private List<String> getKeyValuePair(String line) {
		List<String> pair = new ArrayList<String>();
		String[] keyValuePairs = line.split(ASSIGNMENT);
		if (keyValuePairs.length > 0) {
			pair.add(keyValuePairs[0].trim());
		}
		if (keyValuePairs.length > 1) {
			pair.add(keyValuePairs[1].trim());
		} else {
			pair.add(null);
		}
		return pair;
	}
}
