/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.server;

import java.io.IOException;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;

import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

import raging.goblin.infolog.server.data.InfologStorage;
import raging.goblin.infolog.server.gui.ServerGui;
import raging.goblin.infolog.server.webservice.InfologWebservice;

/**
 * The server application, is the starting point to start an Infolog server application.
 */
public class ServerApplication {

    private static final Logger LOG = Logger.getLogger(ServerApplication.class);
    private static final String GUI_SWITCH = "-g";
    private static final String PORT_SWITCH = "-p";
    private static final String ENDPOINT_SWITCH = "-e";
    private static final int MAX_START_TRY = 5;
    private static final long RETRY_DELAY = 10000;

    public static String ipAddress = getIP();
    public static int port = 8087;
    private static boolean showGui = false;

    private InfologWebservice webservice;

    public boolean startServer() {
        if (InfologStorage.getInstance().start()) {
            webservice = new InfologWebservice();
            return webservice.start();
        }
        return false;
    }

    public void stopServer() {
        webservice.stop();
    }

    public static void main(String[] args) throws IllegalArgumentException, NullPointerException, IOException {
        DOMConfigurator.configureAndWatch("log4j.xml");

        if (args.length % 2 != 0) {
            showUsage();
        }

        try {
            for (int i = 0; i < args.length; i += 2) {
                if (args[i].equals(GUI_SWITCH)) {
                    showGui = Boolean.parseBoolean(args[i + 1]);
                }
                if (args[i].equals(PORT_SWITCH)) {
                    port = Integer.parseInt(args[i + 1]);
                }
                if (args[i].equals(ENDPOINT_SWITCH)) {
                    ipAddress = args[i + 1];
                }
            }
        } catch (Exception e) {
            showUsage();
        }

        if (showGui) {
            ServerGui gui = new ServerGui(new ServerApplication());
            gui.setVisible(true);
        } else {
            ServerApplication serverApplication = new ServerApplication();
            int tryCount = 0;
            while (!serverApplication.startServer() && ++tryCount <= MAX_START_TRY) {
                try {
                    Thread.sleep(RETRY_DELAY);
                } catch (InterruptedException e) {
                    LOG.warn("Retry to start server interrupted: " + e.getMessage());
                }
            }

            while (true) {
                try {
                    Thread.sleep(Long.MAX_VALUE);
                } catch (InterruptedException e) {
                    LOG.warn("Server interrupted: " + e.getMessage());
                }
            }
        }
    }

    private static String getIP() {
        String ipOnly = "";
        try {
            Enumeration<NetworkInterface> nifs = NetworkInterface.getNetworkInterfaces();
            if (nifs == null) {
                LOG.error("No interfaces detected on this machine");
                return null;
            }

            while (nifs.hasMoreElements()) {
                NetworkInterface nif = nifs.nextElement();
                // We ignore subinterfaces - as not yet needed.

                if (!nif.isLoopback() && nif.isUp() && !nif.isVirtual()) {
                    Enumeration<InetAddress> adrs = nif.getInetAddresses();
                    while (adrs.hasMoreElements()) {
                        InetAddress adr = adrs.nextElement();
                        if (adr != null && !adr.isLoopbackAddress()
                            && (nif.isPointToPoint() || !adr.isLinkLocalAddress())) {
                            String adrIP = adr.getHostAddress();
                            if (!adrIP.matches("[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}"))
                                continue;
                            String adrName;
                            if (nif.isPointToPoint()) // Performance issues getting hostname for mobile internet sticks
                                adrName = adrIP;
                            else
                                adrName = adr.getCanonicalHostName();

                            if (!adrName.equals(adrIP))
                                return adrIP;
                            else
                                ipOnly = adrIP;
                        }
                    }
                }
            }
            if (ipOnly.length() == 0) {
                LOG.error("No IP address available");
                return null;
            }
            LOG.debug("Local IP is: " + ipOnly);
            return ipOnly;

        } catch (SocketException e) {
            LOG.error("Unable to read from network interfaces", e);
            return null;
        }
    }

    private static void showUsage() {
        System.out.println("InfologServer - Server part of Infolog program");
        System.out.println("java -jar InfologServer.jar - Starts the server");
        System.out
            .println("java -jar InfologServer.jar -g [true|false] - Starts the server with or without the showGui. Default if off (false).");
        System.out
            .println("java -jar InfologServer.jar -p [portnumber] - Starts the server listening at the defined port number. Default is "
                + port + ".");
        System.out
            .println("java -jar InfologServer.jar -e [endpoint address] - Starts the server with the endpoint at the provided ip or hostname. Default is your external ip address.");
        System.exit(-1);
    }

}
