/*
 * Copyright 2012, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.server.data;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.AnnotationConfiguration;

import raging.goblin.crypt.Crypt;
import raging.goblin.crypt.CryptException;
import raging.goblin.crypt.EncryptionType;
import raging.goblin.crypt.Md5;
import raging.goblin.infolog.server.webservice.ForbiddenException;
import raging.goblin.infolog.sync.data.Infolog;

/**
 * Infolog storage to be used through a webservice.
 */
@SuppressWarnings("deprecation")
public class InfologStorage {

    private static Logger logger = Logger.getLogger(InfologStorage.class);
    private static InfologStorage instance;

    private SessionFactory sessionFactory;

    public static InfologStorage getInstance() {
        if (instance == null) {
            instance = new InfologStorage();
        }
        return instance;
    }

    public boolean start() {
        try {
            AnnotationConfiguration annotationConfiguration = new AnnotationConfiguration();
            annotationConfiguration.configure(new File("hibernate.cfg.xml"));
            sessionFactory = annotationConfiguration.buildSessionFactory();
            getAccount("test");
            return true;
        } catch (Exception e) {
            logger.error("Unable to connect to database", e);
        }
        return false;
    }

    public boolean authenticate(String username, String password, String encryptionType) {
        return getAccount(username, password, encryptionType) != null;
    }

    public void updateInfologs(List<Infolog> infologs, String username, String password, String encryptionType) {
        Account account = getAccount(username, password, encryptionType);
        if (account != null) {
            Session session = sessionFactory.openSession();
            for (Infolog infolog : infologs) {
                updateInfolog(infolog, account, session);
            }
            session.flush();
            session.close();
        }
    }

    private void updateInfolog(Infolog infolog, Account account, Session session) {
        InfologDao infologDao = getInfologDao(infolog.getId());
        infologDao.setAccount(account);
        infologDao.setInfoResponsible(account);
        infologDao.setInfoSubject(infolog.getSubject());
        infologDao.setInfoDes(infolog.getDescription());
        infologDao.setInfoType(infolog.getType());
        infologDao.setInfoDatemodified(infolog.getDatemodified() / 1000);
        session.update(infologDao);
    }

    public List<Infolog> addInfologs(List<Infolog> infologs, String username, String password, String encryptionType)
        throws ForbiddenException {
        Account account = getAccount(username, password, encryptionType);
        if (account != null) {
            Session session = sessionFactory.openSession();
            for (Infolog infolog : infologs) {
                addInfolog(infolog, account, session);
            }
            session.flush();
            session.close();
            return infologs;
        }
        throw new ForbiddenException();
    }

    private void addInfolog(Infolog infolog, Account account, Session session) {
        InfologDao infologDao = InfologTransformer.createInfologDao(infolog);
        infologDao.setAccount(account);
        infologDao.setInfoResponsible(account);
        session.save(infologDao);
        infolog.setId(infologDao.getInfoId());
    }

    public void removeInfologs(List<Infolog> infologs, String username, String password, String encryptionType) {
        Account account = getAccount(username, password, encryptionType);
        if (account != null) {
            Session session = sessionFactory.openSession();
            for (Infolog infolog : infologs) {
                removeInfolog(infolog, account, session);
            }
            session.flush();
            session.close();
        }
    }

    private void removeInfolog(Infolog infolog, Account account, Session session) {
        InfologDao infologDao = getInfologDao(infolog.getId());
        infologDao.setAccount(account);
        session.delete(infologDao);
    }

    public List<Infolog> getInfologs(String username, String password, String encryptionType) throws ForbiddenException {
        Account account = getAccount(username, password, encryptionType);
        if (account != null) {
            List<Infolog> infologs = new ArrayList<Infolog>();
            for (InfologDao infologDao : account.getMyInfologs()) {
                infologs.add(InfologTransformer.createInfolog(infologDao));
            }
            return infologs;
        }
        throw new ForbiddenException();
    }

    private Account getAccount(String username, String password, String encryptionType) {
        Account account = getAccount(username);
        if (password != null && password != "" && account != null) {
            if (validatePassword(password, account.getAccountPwd(), encryptionType)) {
                return account;
            } else {
                logger.info("Wrong username or password provided");
            }
        }
        return null;
    }

    @SuppressWarnings("unchecked")
    private Account getAccount(String username) {
        Session session = sessionFactory.openSession();
        Query query = session.getNamedQuery("Account.findByAccountLid");
        query.setParameter("accountLid", username);
        List<Account> accounts = query.list();
        Account account = null;
        if (accounts.size() == 1) {
            account = accounts.get(0);
        }
        session.flush();
        session.close();
        return account;
    }

    @SuppressWarnings("unchecked")
    private InfologDao getInfologDao(Integer infoId) {
        Session session = sessionFactory.openSession();
        Query query = session.getNamedQuery("InfologDao.findByInfoId");
        query.setParameter("id", infoId);
        List<InfologDao> infologDaos = query.list();
        InfologDao infologDao = null;
        if (infologDaos.size() == 1) {
            infologDao = infologDaos.get(0);
        }
        session.flush();
        session.close();
        return infologDao;
    }

    private boolean validatePassword(String password, String databaseValue, String encryptionType) {
        EncryptionType type = EncryptionType.getEncryptionType(encryptionType);
        String salt = "";

        switch (type) {
        case MD5:
            return Md5.crypt(password).equals(databaseValue);
        case MD5_CRYPT:
            salt = databaseValue.substring(7, 19);
            break;
        case SHA256:
        case SHA512:
            salt = databaseValue.substring(7, 27);
            break;
        }
        try {
            String encryptedPassword = Crypt.crypt(password, salt);
            return databaseValue.equals("{crypt}" + encryptedPassword);
        } catch (CryptException e) {
            logger.error("Unable to encrypt password");
            logger.debug(e);
        }
        return false;
    }

}
