/*
 * Copyright 2013, RagingGoblin <http://raginggoblin.wordpress.com>
 * 
 * This file is part of Infolog.
 *
 *  Infolog is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Infolog is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Infolog.  If not, see <http://www.gnu.org/licenses/>.
 */

package raging.goblin.infolog.sync;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;

import raging.goblin.infolog.sync.data.Infolog;
import raging.goblin.infolog.sync.log.LogLevel;
import raging.goblin.infolog.sync.log.Logger;
import raging.goblin.infolog.sync.webserviceclient.HttpErrorCodeException;

/**
 * The Synchronizer synchronizes two {@link InfologStorage} objects with each other.
 */
public class Synchronizer {

    private Logger logger;

    private InfologStorage localStorage;
    private InfologStorage remoteStorage;
    private String username;
    private String password;
    private String encryptionType;

    public Synchronizer(Logger logger, InfologStorage localStorage, InfologStorage remoteStorage, String username,
        String password, String encryptionType) {
        this.logger = logger;
        this.localStorage = localStorage;
        this.remoteStorage = remoteStorage;
        this.username = username;
        this.password = password;
        this.encryptionType = encryptionType;
    }

    public void synchronize(long lastSyncTime, boolean serverWins) throws JsonParseException, JsonMappingException,
        IOException, HttpErrorCodeException, URISyntaxException {
        if (serverWins) {
            modifiedRemoteInfologs(lastSyncTime);
            modifiedLocalInfologs(lastSyncTime);
        } else {
            modifiedLocalInfologs(lastSyncTime);
            modifiedRemoteInfologs(lastSyncTime);
        }
        newLocalInfologs(lastSyncTime);
        newRemoteInfologs(lastSyncTime);
        removedLocalInfologs(lastSyncTime);
        removedRemoteInfologs(lastSyncTime);
        logger.log(LogLevel.INFO, "Synchronization complete.");
    }

    private void modifiedRemoteInfologs(long lastSyncTime) throws JsonParseException, JsonMappingException,
        IOException, HttpErrorCodeException, URISyntaxException {
        modifiedInfologs(remoteStorage, localStorage, lastSyncTime);
    }

    private void modifiedLocalInfologs(long lastSyncTime) throws JsonParseException, JsonMappingException, IOException,
        HttpErrorCodeException, URISyntaxException {
        modifiedInfologs(localStorage, remoteStorage, lastSyncTime);
    }

    private void removedLocalInfologs(long lastSyncTime) throws JsonParseException, JsonMappingException, IOException,
        HttpErrorCodeException, URISyntaxException {
        removedInfologs(localStorage, remoteStorage, lastSyncTime);
    }

    private void removedRemoteInfologs(long lastSyncTime) throws JsonParseException, JsonMappingException, IOException,
        HttpErrorCodeException, URISyntaxException {
        removedInfologs(remoteStorage, localStorage, lastSyncTime);
    }

    private void newRemoteInfologs(long lastSyncTime) throws JsonParseException, JsonMappingException, IOException,
        HttpErrorCodeException, URISyntaxException {
        newInfologs(remoteStorage, localStorage, lastSyncTime, false);
    }

    private void newLocalInfologs(long lastSyncTime) throws JsonParseException, JsonMappingException, IOException,
        HttpErrorCodeException, URISyntaxException {
        newInfologs(localStorage, remoteStorage, lastSyncTime, true);
    }

    private void modifiedInfologs(InfologStorage withModified, InfologStorage toUpdate, long lastSyncTime)
        throws JsonParseException, JsonMappingException, IOException, HttpErrorCodeException, URISyntaxException {
        List<Infolog> canBeModified = withModified.getInfologs(username, password, encryptionType);
        List<Infolog> reference = toUpdate.getInfologs(username, password, encryptionType);
        List<Infolog> modifiedInfologs = getModifiedInfologs(canBeModified, reference, lastSyncTime);
        if (!modifiedInfologs.isEmpty()) {
            toUpdate.updateInfologs(modifiedInfologs, username, password, encryptionType);
        }
    }

    private void newInfologs(InfologStorage withNew, InfologStorage toUpdate, long lastSyncTime, boolean updateId)
        throws JsonParseException, JsonMappingException, IOException, HttpErrorCodeException, URISyntaxException {
        List<Infolog> canContainNew = withNew.getInfologs(username, password, encryptionType);
        List<Infolog> reference = toUpdate.getInfologs(username, password, encryptionType);
        List<Infolog> newInfologs = getNewInfologs(canContainNew, reference, lastSyncTime);
        if (!newInfologs.isEmpty()) {
            List<Infolog> withNewIds = toUpdate.addInfologs(newInfologs, username, password, encryptionType);
            if (updateId) {
                for (int i = 0; i < newInfologs.size(); i++) {
                    newInfologs.get(i).setId(withNewIds.get(i).getId());
                }
                withNew.updateInfologs(newInfologs, username, password, encryptionType);
            }
        }
    }

    private void removedInfologs(InfologStorage deprived, InfologStorage toUpdate, long lastSyncTime)
        throws JsonParseException, JsonMappingException, IOException, HttpErrorCodeException, URISyntaxException {
        List<Infolog> canBeDeprived = deprived.getInfologs(username, password, encryptionType);
        List<Infolog> reference = toUpdate.getInfologs(username, password, encryptionType);
        List<Infolog> removedInfologs = getRemovedInfologs(canBeDeprived, reference, lastSyncTime);
        if (!removedInfologs.isEmpty()) {
            toUpdate.removeInfologs(removedInfologs, username, password, encryptionType);
        }
    }

    private List<Infolog> getModifiedInfologs(List<Infolog> canBeModified, List<Infolog> reference, long lastSyncTime) {
        List<Infolog> modifiedInfologs = new ArrayList<Infolog>();
        for (Infolog modified : canBeModified) {
            for (Infolog original : reference) {
                if (original.equals(modified) && modified.getDatemodified() > original.getDatemodified()) {
                    modifiedInfologs.add(modified);
                }
            }
        }
        return modifiedInfologs;
    }

    private List<Infolog> getNewInfologs(List<Infolog> canContainNew, List<Infolog> reference, long lastSyncTime) {
        List<Infolog> newInfologs = new ArrayList<Infolog>();
        for (Infolog mightBeNew : canContainNew) {
            if (!reference.contains(mightBeNew) && newerThanLastSync(mightBeNew, lastSyncTime)) {
                newInfologs.add(mightBeNew);
            }
        }
        return newInfologs;
    }

    private List<Infolog> getRemovedInfologs(List<Infolog> canBeDeprived, List<Infolog> reference, long lastSyncTime) {
        List<Infolog> removedInfologs = new ArrayList<Infolog>();
        for (Infolog mightBeRemoved : reference) {
            if (!canBeDeprived.contains(mightBeRemoved) && olderThanLastSync(mightBeRemoved, lastSyncTime)) {
                removedInfologs.add(mightBeRemoved);
            }
        }
        return removedInfologs;
    }

    private boolean newerThanLastSync(Infolog infolog, long lastSyncTime) {
        return infolog.getDatemodified() >= lastSyncTime;
    }

    private boolean olderThanLastSync(Infolog infolog, long lastSyncTime) {
        return !newerThanLastSync(infolog, lastSyncTime);
    }
}
